//
// Copyright 2021, Jesús Pérez Lorenzo
//
use crate::{
  AppRunMode,
	appenv::AppEnv,
};

use AppRunMode::{Pro,Premium};

#[derive(Clone, Debug, Default)]
pub struct AppInfo {
  pub name: String,
	pub webname: String,
  pub version: String,
  pub author: String,
  pub about: String,
  pub usedata: String,
  pub appmode: AppRunMode,
}
impl AppInfo {
	pub fn new(app_name: &str, webname: String, version: String, author: String, about: String) -> Self {
    Self {
      name: format!("{} Server",&app_name),
			webname,
      version,
      author,
      about: format!("{}: {}",&app_name,&about),
      usedata: String::from(""),
      appmode: AppRunMode::default(),
    }
  }
	pub fn load_data(&self,app_env: &AppEnv) -> Self {
    let usedata = match std::fs::read_to_string(format!("{}/{}/.use",&envmnt::get_or("APP_HOME", "."),app_env.get_curr_websrvr_config().home_path)) {
      Ok(res) => res,
      // Err(e) => { println!("Error usedata: {}",e); String::from("")},
      Err(_) => String::from(""),
    };
    Self {
      name: self.name.to_owned(),
			webname: self.webname.to_owned(),
      version: self.version.to_owned(),
      author: self.author.to_owned(),
      about: self.about.to_owned(),
      usedata,
      appmode: self.appmode.to_owned(),
    }
  }
  #[must_use]
  pub fn can_do(&self, target: &str) -> bool {
    match target {
      "encrypt" => matches!(self.appmode, Pro | Premium),
      "files" => matches!(self.appmode, Pro | Premium),
      "user_forms" => matches!(self.appmode, Pro | Premium),
      "templates" => matches!(self.appmode, Pro | Premium),
      "session_state" => matches!(self.appmode, Pro | Premium),
      "support" => matches!(self.appmode, Pro | Premium),
      "update" => matches!(self.appmode, Pro | Premium),
      "deploy" => matches!(self.appmode, Pro | Premium),
      "supervisor" => matches!(self.appmode, Premium),
      _ => {
        println!("Target can do: {} undefined", &target);
        false
      }
    }
  }
}

// #[derive(Clone, Debug)]
// pub struct AppEnv {
//   pub info: AppInfo,
//   pub config: Config,
//   pub debug_level: String,
//   pub appkey: String,
//   pub checked: bool,
//   pub collections: HashMap<String, toml::Value>,
//   pub modules: HashMap<String, Module>,
//   // pub modules: HashMap<String, toml::Value>,
// }
