//
/*! Zterton
*/
// Copyright 2021, Jesús Pérez Lorenzo
//
// use std::collections::HashMap;
pub mod datacontext;
pub mod defs;
pub mod utils;
pub mod lang;
pub mod kloud;

#[macro_use]
extern crate kloud_entries_macro_derive;

use std::fmt;
use serde::{Serialize,Deserialize};
//use async_graphql::{Context, Enum, Interface, SimpleObject, Object, Result};
use async_graphql::{Enum, SimpleObject};
/*
#[derive(Eq, PartialEq, Copy, Clone,Debug, Serialize, Deserialize)]
pub enum Lang {
  EN,
  ES,
  FR,
} 
impl Default for Lang {
  fn default() -> Self {
    Lang::EN
  }
}
*/

#[derive(Eq, PartialEq, Copy, Clone,Debug, Serialize,  Deserialize, Enum)]
pub enum KeyType {
  Primary,
  Foreign,
  Secundary,
  Multiple,
  NotSet,
}
impl Default for KeyType {
  fn default() -> Self {
    KeyType::Primary
  }
} 
impl fmt::Display for KeyType {
  fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
    match self {
    KeyType::Primary => write!(f, "{}", "primary"), 
    KeyType::Foreign => write!(f, "{}", "foreign"),
    KeyType::Secundary => write!(f, "{}", "secundary"),
    KeyType::Multiple => write!(f, "{}", "multiple"),
    KeyType::NotSet => write!(f, "{}", "notset"),
    }
  }
}
impl KeyType {
  #[must_use]
  pub fn get_from(val: &str) -> KeyType {
    match val {
      "primary" => KeyType::Primary,
      "foreign" => KeyType::Foreign,
      "secundary" => KeyType::Secundary,
      "multiple" => KeyType::Multiple,
      _ => KeyType::NotSet,
    }
  }
}

#[derive(Clone, Debug, Serialize, Deserialize, Default, SimpleObject)]
pub struct KeyDef{
  name: String,
  keytype: KeyType,
  coll: String,
  collkey: String,
  priority: u16,
}

#[derive(Eq, PartialEq, Copy, Clone,Debug, Serialize,  Deserialize, Enum)]
pub enum OutFormat {
  Html,
  Json,
  Yaml,
  NotSet,
} 
impl Default for OutFormat {
  fn default() -> Self {
    OutFormat::Json
  }
} 

impl fmt::Display for OutFormat {
  fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
    match self {
    OutFormat::Html => write!(f, "{}", "html"), 
    OutFormat::Yaml => write!(f, "{}", "yaml"),
    OutFormat::Json => write!(f, "{}", "json"),
    OutFormat::NotSet => write!(f, "{}", "notset"),
    }
  }
}
impl OutFormat {
  #[must_use]
  pub fn get_from(val: &str) -> OutFormat {
    match val {
      "html" => OutFormat::Html,
      "yaml" => OutFormat::Yaml,
      "json" => OutFormat::Json,
      _ => OutFormat::NotSet,
    }
  }
}
#[allow(non_camel_case_types)]
#[derive(Eq, PartialEq, Copy, Clone,Debug, Serialize,  Deserialize, Enum)]
pub enum RunEnv {
  Production,
  Development,
  Test,
  NotSet,
} 
impl Default for RunEnv {
  fn default() -> Self {
    RunEnv::Production
  }
} 
impl fmt::Display for RunEnv {
  fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
    match self {
    RunEnv::Production => write!(f, "{}", "production"), 
    RunEnv::Development => write!(f, "{}", "development"),
    RunEnv::Test => write!(f, "{}", "test"),
    RunEnv::NotSet => write!(f, "{}", "notset"),
    }
  }
}
impl RunEnv {
  #[must_use]
  pub fn get_from(val: &str) -> RunEnv {
    match val {
      "production" => RunEnv::Production,
      "development" => RunEnv::Development,
      "test" => RunEnv::Test,
      _ => RunEnv::NotSet,
    }
  }
}

#[derive(Eq, PartialEq, Copy, Clone,Debug, Serialize,  Deserialize, Enum)]
pub enum Platform {
  Ui,
  Backend,
  NotSet,
} 
impl Default for Platform {
  fn default() -> Self {
    Platform::Backend
  }
} 
impl fmt::Display for Platform {
  fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
    match self {
    Platform::Ui=> write!(f, "{}", "ui"), 
    Platform::Backend => write!(f, "{}", "backend"),
    Platform::NotSet => write!(f, "{}", "notset"),
    }
  }
}
impl Platform {
  #[must_use]
  pub fn get_from(val: &str) -> Platform {
    match val {
      "ui" => Platform::Ui,
      "backend" => Platform::Backend,
      _ => Platform::NotSet,
    }
  }
}

#[derive(Eq, PartialEq, Copy, Clone,Debug, Serialize,  Deserialize, Enum)]
pub enum Profile {
  Premium,
  Basic,
  Pro,
  NotSet,
} 
impl Default for Profile {
  fn default() -> Self {
    Profile::Basic
  }
} 
impl fmt::Display for Profile {
  fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
    match self {
    Profile::Premium => write!(f, "{}", "premium"), 
    Profile::Basic => write!(f, "{}", "basic"),
    Profile::Pro => write!(f, "{}", "pro"),
    Profile::NotSet => write!(f, "{}", "notset"),
    }
  }
}
impl Profile {
  #[must_use]
  pub fn get_from(val: &str) -> Profile {
    match val {
      "basic" => Profile::Basic,
      "pro" => Profile::Pro,
      "premium" => Profile::Premium,
      _ => Profile::NotSet,
    }
  }
}

#[derive(Eq, PartialEq, Copy, Clone,Debug, Serialize,  Deserialize, Enum)]
pub enum Permissions {
  Read,
  Write,
  ReadWrite,
  NotSet,
} 
impl Default for Permissions {
  fn default() -> Self {
    Permissions::ReadWrite
  }
} 
impl fmt::Display for Permissions {
  fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
    match self {
    Permissions::Read => write!(f, "{}", "read"), 
    Permissions::Write => write!(f, "{}", "write"),
    Permissions::ReadWrite => write!(f, "{}", "readwrite"),
    Permissions::NotSet => write!(f, "{}", "notset"),
    }
  }
}
impl Permissions {
  #[must_use]
  pub fn get_from(val: &str) -> Permissions {
    match val {
      "read" => Permissions::Read,
      "write" => Permissions::Write,
      "readwrite" => Permissions::ReadWrite,
      _ => Permissions::NotSet,
    }
  }
}
#[derive(Eq, PartialEq, Copy, Clone,Debug, Serialize,  Deserialize, Enum)]
pub enum Auth {
  Allow,
  Denied,
  NotSet,
} 
impl Default for Auth {
  fn default() -> Self {
    Auth::Allow
  }
} 

// #[derive(Clone, Debug, Serialize, Deserialize)]
// pub struct LangItems {
//   pub cell: Option<String>,
// }
// impl Default for LangItems {
//   fn default() -> Self {
//     Self {
//       cell: Some(String::from("cell")),
//     }
//   }
// }
#[derive(Clone, Debug, Serialize, Deserialize, Default,SimpleObject)]
pub struct RunContext {
  pub run_env: RunEnv,
  pub platform: Platform,
}

pub fn set_data_uid() -> String {
  uuid::Uuid::new_v4().to_string()
}

#[cfg(test)]
mod tests {
    #[test]
    fn it_works() {
        assert_eq!(2 + 2, 4);
    }
}

#[cfg(test)]
#[macro_use]
extern crate pretty_assertions;
